@@ -646,7 +646,7 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 	buffer * motdbuf = NULL;
 	int len;
 	struct stat sb;
-	char hushpath[MAXPATHLEN];
+	char *hushpath = NULL;
 #endif
 
 	TRACE(("enter ptycommand"));
@@ -686,8 +686,12 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 #ifdef DO_MOTD
 		if (ses.opts->domotd) {
 			/* don't show the motd if ~/.hushlogin exists */
-			strlcpy(hushpath, ses.authstate.pw->pw_dir, sizeof(hushpath));
-			strlcat(hushpath, "/.hushlogin", sizeof(hushpath));
+
+			/* 11 == strlen("/hushlogin\0") */
+			len = strlen(ses.authstate.pw->pw_dir) + 11; 
+
+			hushpath = m_malloc(len);
+			snprintf(hushpath, len, "%s/hushlogin", ses.authstate.pw->pw_dir);
 
 			if (stat(hushpath, &sb) < 0) {
 				/* more than a screenful is stupid IMHO */
@@ -703,6 +707,7 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 				}
 				buf_free(motdbuf);
 			}
+			m_free(hushpath);
 		}
 #endif /* DO_MOTD */
 
@@ -782,7 +787,7 @@ static void execchild(struct ChanSess *chansess) {
 	/* clear environment */
 	/* if we're debugging using valgrind etc, we need to keep the LD_PRELOAD
 	 * etc. This is hazardous, so should only be used for debugging. */
-#ifndef DEBUG_KEEP_ENV
+#ifndef DEBUG_VALGRIND
 #ifdef HAVE_CLEARENV
 	clearenv();
 #else /* don't HAVE_CLEARENV */
@@ -791,7 +796,7 @@ static void execchild(struct ChanSess *chansess) {
 		environ[0] = NULL;
 	}
 #endif /* HAVE_CLEARENV */
-#endif /* DEBUG_KEEP_ENV */
+#endif /* DEBUG_VALGRIND */
 
 	/* We can only change uid/gid as root ... */
 	if (getuid() == 0) {
@@ -851,9 +856,9 @@ static void execchild(struct ChanSess *chansess) {
 		argv[0] = baseshell;
 	} else {
 		/* a login shell should be "-bash" for "/bin/bash" etc */
-		argv[0] = (char*)m_malloc(strlen(baseshell) + 2); /* 2 for "-" */
-		strcpy(argv[0], "-");
-		strcat(argv[0], baseshell);
+		int len = strlen(baseshell) + 2; /* 2 for "-" */
+		argv[0] = (char*)m_malloc(len);
+		snprintf(argv[0], len, "-%s", baseshell);
 	}
 
 	if (chansess->cmd != NULL) {
