@@ -69,9 +69,13 @@ mp_exptmod (mp_int * G, mp_int * X, mp_int * P, mp_int * Y)
   }
     
   /* if the modulus is odd or dr != 0 use the fast method */
+#ifndef NO_FAST_EXPTMOD
   if (mp_isodd (P) == 1 || dr !=  0) {
     return mp_exptmod_fast (G, X, P, Y, dr);
-  } else {
+  }
+  else
+#endif
+  {
     /* otherwise use the generic Barrett reduction technique */
     return s_mp_exptmod (G, X, P, Y);
   }
