@@ -39,9 +39,14 @@
 #define DSS_PRIV_FILENAME "/etc/dropbear/dropbear_dss_host_key"
 #endif
 #ifndef RSA_PRIV_FILENAME
-#define RSA_PRIV_FILENAME "/etc//dropbear/dropbear_rsa_host_key"
+#define RSA_PRIV_FILENAME "/etc/dropbear/dropbear_rsa_host_key"
 #endif
 
+/* Setting this disables the fast exptmod bignum code. It saves ~5kB, but is
+ * perhaps 20% slower for pubkey operations (it is probably worth experimenting
+ * if you want to use this) */
+/*#define NO_FAST_EXPTMOD*/
+
 /* Enable X11 Forwarding */
 #define ENABLE_X11FWD
 
@@ -62,10 +67,16 @@
 
 /* Integrity - at least one required.
  * RFC Draft requires sha1-hmac, and recommends md5-hmac.
+ *
  * Note: there's no point disabling sha1 to save space, since it's used in the
  * for the random number generator and public-key cryptography anyway.
  * Disabling it here will just stop it from being used as the integrity portion
- * of the ssh protocol */
+ * of the ssh protocol.
+ *
+ * These are also used for key fingerprints in logs (when pubkey auth is used),
+ * MD5 fingerprints are printed by default, however SHA1 fingerprints will be
+ * generated otherwise. This isn't exactly optimal, although sha1 fingerprints
+ * are not too hard go create from pubkeys if required. */
 #define DROPBEAR_SHA1_HMAC
 #define DROPBEAR_MD5_HMAC
 
@@ -144,13 +155,16 @@
 #define SFTPSERVER_PATH "/usr/libexec/sftp-server"
 #endif
 
+/* This is used by the scp binary when used as a client binary */
+#define _PATH_SSH_PROGRAM "/usr/bin/ssh"
+
 /* Multi-purpose binary  configuration - if you want to make the combined
  * binary, first define DROPBEAR_MULTI, and then define which of the three
  * components you want. You should then compile Dropbear with 
  * "make clean; make dropbearmulti". You'll need to install the binary
  * manually, see MULTI for details */
 
-/*#define DROPBEAR_MULTI*/
+/* #define DROPBEAR_MULTI */
 
 /* The three multi binaries: dropbear, dropbearkey, dropbearconvert
  * Comment out these if you don't want some of them */
@@ -164,21 +178,53 @@
  *******************************************************************/
 
 #ifndef DROPBEAR_VERSION
-#define DROPBEAR_VERSION "0.38"
+#define DROPBEAR_VERSION "0.39"
 #endif
+
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
 #define PROGNAME "dropbear"
 
-/* Time to wait before sending reply on fail */
-#define FAIL_SLEEP_TIME 2
+/* Spec recommends after one hour or 1 gigabyte of data. One hour
+ * is a bit too verbose, so we try 8 hours */
+#ifndef KEX_REKEY_TIMEOUT
+#define KEX_REKEY_TIMEOUT (3600 * 8)
+#endif
+#ifndef KEX_REKEY_DATA
+#define KEX_REKEY_DATA (1<<30) /* 2^30 == 1GB, this value must be < INT_MAX */
+#endif
+/* Close connections to clients which haven't authorised after AUTH_TIMEOUT */
+#ifndef AUTH_TIMEOUT
+#define AUTH_TIMEOUT 300 /* we choose 5 minutes */
+#endif
+
+/* Minimum key sizes for DSS and RSA */
+#ifndef MIN_DSS_KEYLEN
+#define MIN_DSS_KEYLEN 512
+#endif
+#ifndef MIN_RSA_KEYLEN
+#define MIN_RSA_KEYLEN 512
+#endif
+
+#define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
+
+#define DEV_URANDOM "/dev/urandom"
+
+/* the number of NAME=VALUE pairs to malloc for environ, if we don't have
+ * the clearenv() function */
+#define ENV_SIZE 100
+
+#define MAX_CMD_LEN 1024 /* max length of a command */
+#define MAX_TERM_LEN 200 /* max length of TERM name */
+
+#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
+
+#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
+								 ipv4 and ipv6 don't count twice */
+
+#define _PATH_TTY "/dev/tty"
 
 /* Timeouts in seconds */
 #define SELECT_TIMEOUT 20
-/* Spec recommends after one hour or 1 gigabyte of data */
-#define KEX_REKEY_TIMEOUT 3600
-#define KEX_REKEY_DATA (1<<30) /* 2^30 == 1GB, this value must be < INT_MAX */
-/* Close connections to clients which haven't authorised after AUTH_TIMEOUT */
-#define AUTH_TIMEOUT 300 /* we choose 5 minutes */
 
 /* success/failure defines */
 #define DROPBEAR_SUCCESS 0
@@ -203,6 +249,8 @@
 #define SHA1_HASH_SIZE 20
 /* SHA512 is 64 bytes == 512 bits */
 #define SHA512_HASH_SIZE 64
+/* MD5 is 16 bytes = 128 bits */
+#define MD5_HASH_SIZE 16
 
 /* largest of MD5 and SHA1 */
 #define MAX_MAC_LEN SHA1_HASH_SIZE
@@ -232,27 +280,9 @@
 #define MAX_TRANS_WINDOW 500000000 /* 500MB is sufficient, stopping overflow */
 #define MAX_TRANS_WIN_INCR 500000000 /* overflow prevention */
 
-#define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
-
 #define MAX_STRING_LEN 1400 /* ~= MAX_PROPOSED_ALGO * MAX_NAME_LEN, also
 							   is the max length for a password etc */
 
-#define DEV_URANDOM "/dev/urandom"
-
-/* the number of NAME=VALUE pairs to malloc for environ, if we don't have
- * the clearenv() function */
-#define ENV_SIZE 100
-
-#define MAX_CMD_LEN 1024 /* max length of a command */
-#define MAX_TERM_LEN 200 /* max length of TERM name */
-
-#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
-
-#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
-								 ipv4 and ipv6 don't count twice */
-
-#define _PATH_TTY "/dev/tty"
-
 /* some quick hacks to reduce the double-negatives above */
 #ifndef ENABLE_TCPFWD
 #define DISABLE_TCPFWD
