@@ -22,71 +22,105 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
-#include "util.h"
+/* Perform RSA operations on data, including reading keys, signing and
+ * verification.
+ *
+ * The format is specified in rfc2437, Applied Cryptography or The Handbook of
+ * Applied Cryptography detail the general algorithm. */
+
+#include "includes.h"
+#include "dbutil.h"
 #include "bignum.h"
 #include "rsa.h"
 #include "buffer.h"
 #include "ssh.h"
 #include "random.h"
 
-#include "libtomcrypt/mycrypt.h"
-
 #ifdef DROPBEAR_RSA 
 
 static mp_int * rsa_pad_em(rsa_key * key,
 		const unsigned char * data, unsigned int len);
 
-/* Load a rsa key from a buffer, initialising the values.
+/* Load a public rsa key from a buffer, initialising the values.
  * The key will have the same format as buf_put_rsa_key.
- * These should be freed with rsa_key_free */
+ * These should be freed with rsa_key_free.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_rsa_pub_key(buffer* buf, rsa_key *key) {
 
 	TRACE(("enter buf_get_rsa_pub_key"));
 	assert(key != NULL);
 	key->e = m_malloc(sizeof(mp_int));
-	m_mp_init(key->e);
 	key->n = m_malloc(sizeof(mp_int));
-	m_mp_init(key->n);
+	m_mp_init_multi(key->e, key->n, NULL);
 	key->d = NULL;
+	key->p = NULL;
+	key->q = NULL;
 
 	buf_incrpos(buf, 4+SSH_SIGNKEY_RSA_LEN); /* int + "ssh-rsa" */
 
-	if (buf_getmpint(buf, key->e) != 0
-	 || buf_getmpint(buf, key->n) != 0) {
+	if (buf_getmpint(buf, key->e) == DROPBEAR_FAILURE
+	 || buf_getmpint(buf, key->n) == DROPBEAR_FAILURE) {
 		TRACE(("leave buf_get_rsa_pub_key: failure"));
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
+
+	if (mp_count_bits(key->n) < MIN_RSA_KEYLEN) {
+		dropbear_log(LOG_WARNING, "rsa key too short");
+		return DROPBEAR_FAILURE;
+	}
+
 	TRACE(("leave buf_get_rsa_pub_key: success"));
-	return 0;
+	return DROPBEAR_SUCCESS;
 
 }
 
-/* same as buf_get_rsa_pub_key, but reads a private "x" key at the end.
- * Loads a private rsa key from a buffer */
+/* Same as buf_get_rsa_pub_key, but reads a private "x" key at the end.
+ * Loads a private rsa key from a buffer
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_rsa_priv_key(buffer* buf, rsa_key *key) {
 
-	int ret;
 	assert(key != NULL);
 
 	TRACE(("enter buf_get_rsa_priv_key"));
 
-	ret = buf_get_rsa_pub_key(buf, key);
-	if (ret != 0) {
-		TRACE(("leave buf_get_rsa_priv_key: ret != 0"));
-		return ret;
+	if (buf_get_rsa_pub_key(buf, key) == DROPBEAR_FAILURE) {
+		TRACE(("leave buf_get_rsa_priv_key: pub: ret == DROPBEAR_FAILURE"));
+		return DROPBEAR_FAILURE;
 	}
 
 	key->d = m_malloc(sizeof(mp_int));
 	m_mp_init(key->d);
-	ret = buf_getmpint(buf, key->d);
+	if (buf_getmpint(buf, key->d) == DROPBEAR_FAILURE) {
+		TRACE(("leave buf_get_rsa_priv_key: d: ret == DROPBEAR_FAILURE"));
+		return DROPBEAR_FAILURE;
+	}
+
+	/* old Dropbear private keys didn't keep p and q, so we will ignore them*/
+	if (buf->pos == buf->len) {
+		key->p = NULL;
+		key->q = NULL;
+	} else {
+		key->p = m_malloc(sizeof(mp_int));
+		key->q = m_malloc(sizeof(mp_int));
+		m_mp_init_multi(key->p, key->q, NULL);
+
+		if (buf_getmpint(buf, key->p) == DROPBEAR_FAILURE) {
+			TRACE(("leave buf_get_rsa_priv_key: p: ret == DROPBEAR_FAILURE"));
+			return DROPBEAR_FAILURE;
+		}
+
+		if (buf_getmpint(buf, key->q) == DROPBEAR_FAILURE) {
+			TRACE(("leave buf_get_rsa_priv_key: q: ret == DROPBEAR_FAILURE"));
+			return DROPBEAR_FAILURE;
+		}
+	}
 
 	TRACE(("leave buf_get_rsa_priv_key"));
-	return ret;
+	return DROPBEAR_SUCCESS;
 }
 	
 
-/* clear and free the memory used by a public key */
+/* Clear and free the memory used by a public or private key */
 void rsa_key_free(rsa_key *key) {
 
 	TRACE(("enter rsa_key_free"));
@@ -107,11 +141,19 @@ void rsa_key_free(rsa_key *key) {
 		 mp_clear(key->n);
 		 m_free(key->n);
 	}
+	if (key->p) {
+		mp_clear(key->p);
+		m_free(key->p);
+	}
+	if (key->q) {
+		mp_clear(key->q);
+		m_free(key->q);
+	}
 	m_free(key);
 	TRACE(("leave rsa_key_free"));
 }
 
-/* put the rsa key into the buffer in the required format:
+/* Put the public rsa key into the buffer in the required format:
  *
  * string	"ssh-rsa"
  * mp_int	e
@@ -139,26 +181,36 @@ void buf_put_rsa_priv_key(buffer* buf, rsa_key *key) {
 	buf_put_rsa_pub_key(buf, key);
 	buf_putmpint(buf, key->d);
 
+	/* new versions have p and q, old versions don't */
+	if (key->p) {
+		buf_putmpint(buf, key->p);
+	}
+	if (key->q) {
+		buf_putmpint(buf, key->q);
+	}
+
+
 	TRACE(("leave buf_put_rsa_priv_key"));
 
 }
 
 #ifdef DROPBEAR_SIGNKEY_VERIFY
+/* Verify a signature in buf, made on data by the key given.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_rsa_verify(buffer * buf, rsa_key *key, const unsigned char* data,
 		unsigned int len) {
 
 	unsigned int slen;
 	mp_int rsa_s, rsa_mdash;
 	mp_int *rsa_em = NULL;
-	int ret = 0;
+	int ret = DROPBEAR_FAILURE;
 
 	assert(key != NULL);
 
-	m_mp_init(&rsa_mdash);
-	m_mp_init(&rsa_s);
+	m_mp_init_multi(&rsa_mdash, &rsa_s, NULL);
 
 	slen = buf_getint(buf);
-	if (slen != mp_unsigned_bin_size(key->n)) {
+	if (slen != (unsigned int)mp_unsigned_bin_size(key->n)) {
 		TRACE(("bad size"));
 		goto out;
 	}
@@ -168,6 +220,12 @@ int buf_rsa_verify(buffer * buf, rsa_key *key, const unsigned char* data,
 		goto out;
 	}
 
+	/* check that s <= n-1 */
+	if (mp_cmp(&rsa_s, key->n) != MP_LT) {
+		TRACE(("s > n-1"));
+		goto out;
+	}
+
 	/* create the magic PKCS padded value */
 	rsa_em = rsa_pad_em(key, data, len);
 
@@ -175,22 +233,20 @@ int buf_rsa_verify(buffer * buf, rsa_key *key, const unsigned char* data,
 		goto out;
 	}
 
-	if (mp_cmp(rsa_em, &rsa_mdash) == 0) {
+	if (mp_cmp(rsa_em, &rsa_mdash) == MP_EQ) {
 		/* signature is valid */
-		ret = 1;
+		ret = DROPBEAR_SUCCESS;
 	}
 
 out:
-	mp_clear(rsa_em);
+	mp_clear_multi(rsa_em, &rsa_mdash, &rsa_s, NULL);
 	m_free(rsa_em);
-	mp_clear(&rsa_mdash);
-	mp_clear(&rsa_s);
 	return ret;
 
 }
 #endif /* DROPBEAR_SIGNKEY_VERIFY */
 
-/* sign the data presented with key, writing the signature contents
+/* Sign the data presented with key, writing the signature contents
  * to the buffer */
 void buf_put_rsa_sign(buffer* buf, rsa_key *key, const unsigned char* data,
 		unsigned int len) {
@@ -234,7 +290,7 @@ void buf_put_rsa_sign(buffer* buf, rsa_key *key, const unsigned char* data,
 	buf_incrwritepos(buf, ssize);
 	mp_clear(&rsa_s);
 
-#ifdef DEBUG_RSA
+#if defined(DEBUG_RSA) && defined(DEBUG_TRACE)
 	printhex(buf->data, buf->len);
 #endif
 	
@@ -242,7 +298,7 @@ void buf_put_rsa_sign(buffer* buf, rsa_key *key, const unsigned char* data,
 	TRACE(("leave buf_put_rsa_sign"));
 }
 
-/* creates the message value as expected by PKCS, see rfc2437 etc */
+/* Creates the message value as expected by PKCS, see rfc2437 etc */
 /* format to be padded to is:
  * EM = 01 | FF* | 00 | prefix | hash
  *
@@ -255,37 +311,41 @@ void buf_put_rsa_sign(buffer* buf, rsa_key *key, const unsigned char* data,
 static mp_int * rsa_pad_em(rsa_key * key,
 		const unsigned char * data, unsigned int len) {
 
+	/* ASN1 designator (including the 0x00 preceding) */
 	const char rsa_asn1_magic[] = 
-		{0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 
+		{0x00, 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 
 		 0x0e, 0x03, 0x02, 0x1a, 0x05, 0x00, 0x04, 0x14};
-#define RSA_ASN1_MAGIC_LEN 15
+#define RSA_ASN1_MAGIC_LEN 16
 	buffer * rsa_EM;
 	hash_state hs;
 	unsigned int nsize;
 	mp_int * rsa_em;
-	unsigned int i;
 	
 	assert(key != NULL);
 	assert(data != NULL);
 	nsize = mp_unsigned_bin_size(key->n);
 
 	rsa_EM = buf_new(nsize-1);
+	/* type byte */
 	buf_putbyte(rsa_EM, 0x01);
-	for (i = 0; i < (nsize-1)-SHA1_HASH_SIZE-2-RSA_ASN1_MAGIC_LEN; i++) {
+	/* Padding with 0xFF bytes */
+	while(rsa_EM->pos != rsa_EM->size - RSA_ASN1_MAGIC_LEN - SHA1_HASH_SIZE) {
 		buf_putbyte(rsa_EM, 0xff);
 	}
-	buf_putbyte(rsa_EM, 0x00);
+	/* Magic ASN1 stuff */
 	memcpy(buf_getwriteptr(rsa_EM, RSA_ASN1_MAGIC_LEN),
 			rsa_asn1_magic, RSA_ASN1_MAGIC_LEN);
 	buf_incrwritepos(rsa_EM, RSA_ASN1_MAGIC_LEN);
 
-	/* hash the data */
+	/* The hash of the data */
 	sha1_init(&hs);
 	sha1_process(&hs, data, len);
 	sha1_done(&hs, buf_getwriteptr(rsa_EM, SHA1_HASH_SIZE));
 	buf_incrwritepos(rsa_EM, SHA1_HASH_SIZE);
+
 	assert(rsa_EM->pos == rsa_EM->size);
 
+	/* Create the mp_int from the encoded bytes */
 	buf_setpos(rsa_EM, 0);
 	rsa_em = (mp_int*)m_malloc(sizeof(mp_int));
 	m_mp_init(rsa_em);
